# AUTO GENERATED FILE - DO NOT EDIT

from dash.development.base_component import Component, _explicitize_args


class DashMarvinJS(Component):
    """A DashMarvinJS component.


Keyword arguments:

- id (string; optional):
    The ID used to identify this component in Dash callbacks.

- input (dict; optional):
    Structure and selected atoms/bonds.

    `input` is a dict with keys:

    - atoms (string; optional)

    - bonds (string; optional)

    - structure (string; optional)

- marvin_button (dict; default {    'name' : 'Upload',    'image-url' : 'data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAIAAAACACAMAAAD04JH5AAAArlBMVEUAAAAAAAA' +                  'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA' +                  'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA' +                  'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAABeyFOlAAAAAXRSTlMAQObYZgAAAAFiS0' +                  'dEAIgFHUgAAAAJcEhZcwAACcUAAAnFAYeExPQAAAAHdElNRQfjCQwVADLJ+C5eAAABUklEQVR42u3ZQWrDMBC' +                  'F4TlMIFun2JD//hdLodBFLamWrMlL2jc7WWbmi2LkEY44FRsQulj4ClV9vkNdXyMArQC0AtAKQCsArQC0AtAK' +                  'QCsArQC0AtAKQCsArQC0AtAKQCsArWBfojpIESz7AvVRhqCQvjGcD9gK2Vvj6YJS7uaFXECtLX8WoH4uSHwIC' +                  'olpX1vTj2HFX5u3Eezzlpc7cTP+ua61/zth/V/zcGqAAQZkAY63ea25bbRZ7Ok06zMfzSzXevlLV69bnViGW2' +                  'bmAH5Ncxvv/I8AtuFTwyTAgTT3KUfvEwDeFrDOeQjHASEGHN2IkgBrx1Z8GpDxMjLAAAMMMMAAAwwwwAADDDD' +                  'AAAMMMODvAaLzO3EmIDSAGLt7IqDvO3EKoCcMMMAAAwwwwAADDDDAgPcCZIcBBrwmIP4RoLJTXNWAUNePuInr' +                  'f8b92eUfelqXBAH/Tb4AAAAASUVORK5CYII=',    'toolbar' : 'N'}):
    Button config of MarvinJS iframe.

    `marvin_button` is a dict with keys:

    - image-url (string; optional)

    - name (string; optional)

    - toolbar (string; optional)

- marvin_height (string; default '450'):
    Height of MarvinJS iframe.

- marvin_license (dict; default {    'url': '/license.cxl',    'is_dynamic': False}):
    License location.

    `marvin_license` is a dict with keys:

    - is_dynamic (boolean; optional)

    - url (string; optional)

- marvin_services (dict; default {    'molconvertws': '/importer'}):
    Structure import backend.

    `marvin_services` is a dict with keys:

    - molconvertws (string; optional)

- marvin_url (string; required):
    A URL of MarvinJS iframe.

- marvin_width (string; default '900'):
    Width of MarvinJS iframe.

- output (dict; optional):
    Structure from backend for rendering.

    `output` is a dict with keys:

    - atoms (string; optional)

    - bonds (string; optional)

    - structure (string; optional)"""
    @_explicitize_args
    def __init__(self, id=Component.UNDEFINED, marvin_url=Component.REQUIRED, marvin_width=Component.UNDEFINED, marvin_height=Component.UNDEFINED, marvin_button=Component.UNDEFINED, marvin_services=Component.UNDEFINED, marvin_license=Component.UNDEFINED, output=Component.UNDEFINED, input=Component.UNDEFINED, **kwargs):
        self._prop_names = ['id', 'input', 'marvin_button', 'marvin_height', 'marvin_license', 'marvin_services', 'marvin_url', 'marvin_width', 'output']
        self._type = 'DashMarvinJS'
        self._namespace = 'dash_marvinjs'
        self._valid_wildcard_attributes =            []
        self.available_properties = ['id', 'input', 'marvin_button', 'marvin_height', 'marvin_license', 'marvin_services', 'marvin_url', 'marvin_width', 'output']
        self.available_wildcard_properties =            []
        _explicit_args = kwargs.pop('_explicit_args')
        _locals = locals()
        _locals.update(kwargs)  # For wildcard attrs and excess named props
        args = {k: _locals[k] for k in _explicit_args if k != 'children'}
        for k in ['marvin_url']:
            if k not in args:
                raise TypeError(
                    'Required argument `' + k + '` was not specified.')
        super(DashMarvinJS, self).__init__(**args)



from chython import MRVWrite, smiles, SDFRead, RDFRead, SMILESRead
from io import StringIO
from itertools import chain
from flask import request


def importer(app, route='/importer', skip_mapping=True):
    """
    SMILES, SDF, RDF importer

    :param app: dash app
    :param route: url for binding
    :param skip_mapping: skip import of atom to atom mapping
    """
    @app.server.route(route, methods=['POST'])
    def callback():
        data = request.json['structure']
        for r in (SMILESRead, RDFRead, SDFRead):
            for s in r(StringIO(data)):
                s.clean2d()
                with StringIO() as f:
                    with MRVWrite(f) as o:
                        o.write(s, skip_mapping=skip_mapping)
                    return {'structure': f.getvalue()}
        return {'structure': None}


__all__ = ['importer']



from chython import MRVRead, MRVWrite, ReactionContainer, MoleculeContainer
from dataclasses import dataclass
from functools import wraps
from io import BytesIO, StringIO
from typing import Optional, Union, List, Tuple


@dataclass
class Input:
    structure: Union[MoleculeContainer, ReactionContainer, None]
    atoms: Union[List[int], List[Tuple[int, int]], None] = None
    bonds: Union[List[Tuple[int, int]], List[Tuple[int, int, int]], None] = None


def prepare_input(idx: int = 0):
    """
    Decorate callback input. Converts MRV string into chython structures and process selected atoms and bonds.
    None input propagated as is.

    Output format:

    (MoleculeContainer, [selected atoms numbers], [(bonds atoms numbers, pairs)])
     or
    (ReactionContainer, [(molecule index, selected atom number)], [(molecule index, bonds atoms numbers, pairs)])

    :param idx: index of the MJS input in Dash callback.
    """
    def d(fn):
        @wraps(fn)
        def w(*args):
            if (ai := args[idx]) is not None:
                sa = [int(x) for x in ai['atoms'].split(',')] if ai['atoms'] else []
                sb = [[int(x) for x in x.split('-')] for x in ai['bonds'].split(',')] if ai['bonds'] else []

                args = list(args)
                with BytesIO(ai['structure'].encode()) as f, MRVRead(f, ignore=True) as i:
                    try:
                        s = next(i)
                    except StopIteration:
                        args[idx] = None
                    else:
                        if isinstance(s, ReactionContainer):
                            mp = dict(enumerate(((y, x) for y, x in enumerate(s.molecules()) for x in x), 1))
                            sb = [(*mp[x], mp[y][1]) for x, y in sb]
                        else:  # molecule
                            mp = dict(enumerate(s, 1))
                            sb = [(mp[x], mp[y]) for x, y in sb]
                        args[idx] = Input(s, [mp[x] for x in sa], sb)
            return fn(*args)
        return w

    if callable(idx):  # decorated function
        f, idx = idx, 0
        return d(f)
    return d


def prepare_output(idx: Optional[int] = None, skip_mapping: bool = True):
    """
    Decorate callback output. Converts chython structures into MRV string. Use None for canvas cleaning.

    :param idx: index of the chython object in the output tuple for multiple outputs or None for single output.
    :param skip_mapping: remove atom numbers.
    """
    def d(fn):
        @wraps(fn)
        def w(*args):
            out = fn(*args)
            s = out if idx is None else out[idx]
            if s is not None:
                if isinstance(s, (MoleculeContainer, ReactionContainer)):
                    s = Input(s)
                elif isinstance(s, Input):
                    if s.structure is None:
                        s = {'structure': None}
                        if idx is None:
                            return s
                        out = list(out)
                        out[idx] = s
                        return tuple(out)
                    elif isinstance(s.structure, MoleculeContainer):
                        mp = {x: n for n, x in enumerate(s.structure, 1)}
                        if (bonds := s.bonds) is not None:
                            bonds = ','.join(f'{mp[x]}-{mp[y]}' for x, y in bonds)
                    elif isinstance(s.structure, ReactionContainer):
                        mp = {nx: m for m, nx in
                              enumerate(((n, x) for n, x in enumerate(s.structure.molecules()) for x in x), 1)}
                        if (bonds := s.bonds) is not None:
                            bonds = ','.join(f'{mp[(n, x)]}-{mp[(n, y)]}' for n, x, y in bonds)
                    else:
                        raise TypeError('MoleculeContainer, ReactionContainer or None expected')

                    if (atoms := s.atoms) is not None:
                        atoms = ','.join(str(mp[x]) for x in atoms)
                    s = Input(s.structure, atoms, bonds)
                else:
                    raise TypeError('dash_marvinjs.Input or MoleculeContainer or ReactionContainer expected')
                with StringIO() as f:
                    with MRVWrite(f) as o:
                        o.write(s.structure, skip_mapping=skip_mapping)
                    s = {'structure': f.getvalue(), 'atoms': s.atoms, 'bonds': s.bonds}
            if idx is None:
                return s
            out = list(out)
            out[idx] = s
            return tuple(out)
        return w

    if callable(idx):  # decorated function
        f, idx = idx, None
        return d(f)
    return d


__all__ = ['prepare_input', 'prepare_output', 'Input']
